==> bodhi-cli-0.9.13.tgz: package/lib/stores/registration.js <==
module.exports = function(client, api, views){

    var TYPE     = "Store";
    var RESOURCE = ['resources', TYPE].join('/');
    var prompt   = require('prompt');
    var utils    = require('./storeUtils');
    var lodash   = require('lodash');

    var inspect = require('eyes').inspector({
        styles: {                 // Styles applied to stdout
            all:     'cyan',      // Overall style applied to everything
            label:   'underline', // Inspection labels, like 'array' in `array: [1, 2, 3]`
            other:   'inverted',  // Objects which don't have a literal representation, such as functions
            key:     'bold',      // The keys in object literals, like 'a' in `{a: 1}`
            special: 'red',      // null, undefined...
            string:  'yellow',
            number:  'green',
            bool:    'green',      // true false
            regexp:  'green'      // /\d+/
        },
        pretty: true,             // Indent object literals
        hideFunctions: false,     // Don't output functions at all
        stream: process.stdout,   // Stream to write to, or null
        maxLength: 4048           // Truncate output if longer
    });

    function register(options){

        prompt.message = TYPE;
        prompt.delimiter = ":".green;

        var schema = {
            properties: {
                name: {
                    description: 'Enter a url slug for the store'.green,
                    pattern: /^[a-zA-Z\s\-]+$/,
                    message: 'Module must be only letters or dashes and represents a readable key for the store',
                    required: true
                },
                store_number: {
                    type       : 'string',
                    description: 'what is the formal identifier for the store used by organization'.green,
                    required: true
                },
                display_name: {
                    type   : 'string',
                    description: 'what name should be displayed in user interfaces'.green,
                    required: true
                },
                street_address: {
                    type       : 'string',
                    description: 'what is the street address'.green
                },
                locality: {
                    type       : 'string',
                    description: 'what is the locality (city)'.green
                },
                region: {
                    type       : 'string',
                    description: 'what is the region (state)'.green
                },
                postal_code: {
                    type       : 'string',
                    description: 'what is the postal code'.green
                },
                country: {
                    type       : 'string',
                    description: 'what is the country'.green,
                    default    : 'US'
                }
            }
        };

        prompt.start();

        prompt.get(schema, function (err, representation) {
            console.log('Creating new job with name %s', representation.name);
            var address = {};
            ['street_address', 'locality', 'region', 'postal_code', 'country']
                .forEach(function(element, index, array){
                    if(representation[element]){
                        address[element] = representation[element];
                        delete representation[element];
                    };
                });

            if(!lodash.isEmpty(address)){
                representation.address = address;
            }

            api.post(RESOURCE, representation, function(err, json, ctx){
                if(err){
                    console.log('Failed to create the store: %s', err.messsage);
                } else {
                    console.log('Created the store at %s', api.resolve(ctx.headers.location));
                }
            });
        });
    }

    function create(options){
        var store = {};
        store.name = options.name;
        store.store_number = '' + options.number;
        api.post(RESOURCE, store, function(err, json, ctx){
            if(err){
                console.log('Failed to create the store: %s', err.messsage);
            } else {
                console.log('Created the store at %s', api.resolve(ctx.headers.location));
            }
        })
    }

    function list(){
        client.get(RESOURCE, function(err, json, ctx){
            if(err){
                console.log('Unable to locate stores: %s', err.messsage);
            } else {
                console.log(inspect(json.body));
            }
        });
    }

    function fetchAndUpdate(resource, update){
        api.fetchAndUpdate(resource, update, function(err, json, ctx){
            if(err){
                console.log('Unable to update the store %s, : %s', resource, err);
                console.log(err);
            } else {
                console.log('Updated the store %s', resource);
            }
        });
    }

    function tag(tags, options){
        options = options || {};
        fetchAndUpdate(
            utils.createResource(TYPE, null, options),
            function _tag(res){
                res.tags = res.tags || [];
                res.tags = lodash.union(res.tags, tags);
                return res;
            }
        );
    }

    function untag(tags, options){
        options = options || {};
        fetchAndUpdate(
            utils.createResource(TYPE, null, options),
            function _tag(res){
                res.tags = res.tags || [];
                res.tags = lodash.difference(res.tags, tags);
                return res;
            }
        );
    }

    function activate(options){
        return setStatus('active'  , options)
    }
    function deactivate(options){
        return setStatus('inactive', options)
    }

    function setStatus(status, options){
        options = options || {};
        fetchAndUpdate(
            utils.createResource(TYPE, null, options),
            function _update(res){
                if(status) {
                    res.status = status || null
                }
                return res;
            }
        );
    }

    function arrayObjectIndexOf(myArray, searchTerm, property) {
        for(var i = 0, len = myArray.length; i < len; i++) {
            if (myArray[i][property] === searchTerm) return i;
        }
        return -1;
    }

    function link(options){
        options = options || {};
        fetchAndUpdate(
            utils.createResource(TYPE, null, options),
            function _update(res){
                if(options.key && options.value){
                    res.external_ids = res.external_ids || [];

                    //entry
                    var entry = {
                        key  : options.key,
                        value: '' + options.value
                    };

                    var idx = arrayObjectIndexOf(res.external_ids, entry.key, 'key');
                    if(idx < 0){
                        Array.prototype.push.apply(res.external_ids, [entry]);
                    } else {
                        res.external_ids[idx] = entry;
                    }
                }
                return res;
            }
        );
    }

    function set(options){
        options = options || {};
        fetchAndUpdate(
            utils.createResource(TYPE, null, options),
            function _update(res){
                if(options.key && options.value){
                    res[options.key] = options.value;
                }
                return res;
            }
        );
    }

    function unlink(options){
        options = options || {};
        fetchAndUpdate(
            utils.createResource(TYPE, null, options),
            function _update(res){
                if(options.key){
                    res.external_ids = res.external_ids || [];
                    var idx = arrayObjectIndexOf(res.external_ids, options.key, 'key');
                    if(idx >= 0){
                        res.external_ids.splice(idx, 1);
                    }
                }
                return res;
            }
        );
    }

    function fetch(id, options){
        var resource = utils.createResource(TYPE, id, options);
        api.fetch(resource, function(err, json, ctx){
            if(err){
                console.log('Unable to find the stora %s, : %s', resource, err.messsage);
            } else {

                var out = null;
                if(options.color){
                    out = inspect(json)
                } else {
                    out = JSON.stringify(json, null, '  ');
                }
                console.log(out);
            }
        });
    }

    //This is a more critical
    function unregister(id, options){
        var resource = utils.createResource(TYPE, id, options);
        console.log('deleting %s', resource);
        api.fetchAndRemove(utils.createResource(TYPE, id, options), function(err, json, ctx){
            if(err){
                console.log('unregister request failed: %s', err.message)
            } else {
                console.log('store %s removed', resource)
            }
        });
    }

    return {
        register  : register,       //done
        create    : create,         //done
        activate  : activate,       //done
        deactivate: deactivate,     //done
        setStatus : setStatus,      //done
        list      : list,           //done
        fetch     : fetch,          //done
        link      : link,           //done
        unlink    : unlink,         //done
        tag       : tag,            //done
        untag     : untag,          //done
        unregister: unregister      //done
    }

};

==> bodhi-cli-0.9.15.tgz: package/lib/stores/registration.js <==
module.exports = function(client, api, views){

    var TYPE     = "Store";
    var RESOURCE = ['resources', TYPE].join('/');
    var prompt   = require('prompt');
    var utils    = require('./storeUtils');
    var lodash   = require('lodash');

    var inspect = require('eyes').inspector({
        styles: {                 // Styles applied to stdout
            all:     'cyan',      // Overall style applied to everything
            label:   'underline', // Inspection labels, like 'array' in `array: [1, 2, 3]`
            other:   'inverted',  // Objects which don't have a literal representation, such as functions
            key:     'bold',      // The keys in object literals, like 'a' in `{a: 1}`
            special: 'red',      // null, undefined...
            string:  'yellow',
            number:  'green',
            bool:    'green',      // true false
            regexp:  'green'      // /\d+/
        },
        pretty: true,             // Indent object literals
        hideFunctions: false,     // Don't output functions at all
        stream: process.stdout,   // Stream to write to, or null
        maxLength: 4048           // Truncate output if longer
    });

    function register(options){

        prompt.message = TYPE;
        prompt.delimiter = ":".green;

        var schema = {
            properties: {
                name: {
                    description: 'Enter a url slug for the store'.green,
                    pattern: /^[a-zA-Z\s\-]+$/,
                    message: 'Module must be only letters or dashes and represents a readable key for the store',
                    required: true
                },
                store_number: {
                    type       : 'string',
                    description: 'what is the formal identifier for the store used by organization'.green,
                    required: true
                },
                display_name: {
                    type   : 'string',
                    description: 'what name should be displayed in user interfaces'.green,
                    required: true
                },
                street_address: {
                    type       : 'string',
                    description: 'what is the street address'.green
                },
                locality: {
                    type       : 'string',
                    description: 'what is the locality (city)'.green
                },
                region: {
                    type       : 'string',
                    description: 'what is the region (state)'.green
                },
                postal_code: {
                    type       : 'string',
                    description: 'what is the postal code'.green
                },
                country: {
                    type       : 'string',
                    description: 'what is the country'.green,
                    default    : 'US'
                }
            }
        };

        prompt.start();

        prompt.get(schema, function (err, representation) {
            console.log('Creating new job with name %s', representation.name);
            var address = {};
            ['street_address', 'locality', 'region', 'postal_code', 'country']
                .forEach(function(element, index, array){
                    if(representation[element]){
                        address[element] = representation[element];
                        delete representation[element];
                    };
                });

            if(!lodash.isEmpty(address)){
                representation.address = address;
            }

            api.post(RESOURCE, representation, function(err, json, ctx){
                if(err){
                    console.log('Failed to create the store: %s', err);
                } else {
                    console.log('Created the store at %s', api.resolve(ctx.headers.location));
                }
            });
        });
    }

    function create(options){
        var store = {};
        store.name = options.name;
        store.store_number = '' + options.number;
        api.post(RESOURCE, store, function(err, json, ctx){
            if(err){
                console.log('Failed to create the store: %s', err);
            } else {
                console.log('Created the store at %s', api.resolve(ctx.headers.location));
            }
        })
    }

    function list(){
        client.get(RESOURCE, function(err, json, ctx){
            if(err){
                console.log('Unable to locate stores: %s', err);
            } else {
                console.log(inspect(json.body));
            }
        });
    }

    function fetchAndUpdate(resource, update){
        api.fetchAndUpdate(resource, update, function(err, json, ctx){
            if(err){
                console.log('Unable to update the store %s, : %s', resource, err);
                console.log(err);
            } else {
                console.log('Updated the store %s', resource);
            }
        });
    }

    function tag(tags, options){
        options = options || {};
        fetchAndUpdate(
            utils.createResource(TYPE, null, options),
            function _tag(res){
                res.tags = res.tags || [];
                res.tags = lodash.union(res.tags, tags);
                return res;
            }
        );
    }

    function untag(tags, options){
        options = options || {};
        fetchAndUpdate(
            utils.createResource(TYPE, null, options),
            function _tag(res){
                res.tags = res.tags || [];
                res.tags = lodash.difference(res.tags, tags);
                return res;
            }
        );
    }

    function activate(options){
        return setStatus('active'  , options)
    }
    function deactivate(options){
        return setStatus('inactive', options)
    }

    function setStatus(status, options){
        options = options || {};
        fetchAndUpdate(
            utils.createResource(TYPE, null, options),
            function _update(res){
                if(status) {
                    res.status = status || null
                }
                return res;
            }
        );
    }

    function arrayObjectIndexOf(myArray, searchTerm, property) {
        for(var i = 0, len = myArray.length; i < len; i++) {
            if (myArray[i][property] === searchTerm) return i;
        }
        return -1;
    }

    function link(options){
        options = options || {};
        fetchAndUpdate(
            utils.createResource(TYPE, null, options),
            function _update(res){
                if(options.key && options.value){
                    res.external_ids = res.external_ids || [];

                    //entry
                    var entry = {
                        key  : options.key,
                        value: '' + options.value
                    };

                    var idx = arrayObjectIndexOf(res.external_ids, entry.key, 'key');
                    if(idx < 0){
                        Array.prototype.push.apply(res.external_ids, [entry]);
                    } else {
                        res.external_ids[idx] = entry;
                    }
                }
                return res;
            }
        );
    }

    function set(options){
        options = options || {};
        fetchAndUpdate(
            utils.createResource(TYPE, null, options),
            function _update(res){
                if(options.key && options.value){
                    res[options.key] = options.value;
                }
                return res;
            }
        );
    }

    function unlink(options){
        options = options || {};
        fetchAndUpdate(
            utils.createResource(TYPE, null, options),
            function _update(res){
                if(options.key){
                    res.external_ids = res.external_ids || [];
                    var idx = arrayObjectIndexOf(res.external_ids, options.key, 'key');
                    if(idx >= 0){
                        res.external_ids.splice(idx, 1);
                    }
                }
                return res;
            }
        );
    }

    function fetch(id, options){
        var resource = utils.createResource(TYPE, id, options);
        api.fetch(resource, function(err, json, ctx){
            if(err){
                console.log('Unable to find the stora %s, : %s', resource, err.messsage);
            } else {

                var out = null;
                if(options.color){
                    out = inspect(json)
                } else {
                    out = JSON.stringify(json, null, '  ');
                }
                console.log(out);
            }
        });
    }

    //This is a more critical
    function unregister(id, options){
        var resource = utils.createResource(TYPE, id, options);
        console.log('deleting %s', resource);
        api.fetchAndRemove(utils.createResource(TYPE, id, options), function(err, json, ctx){
            if(err){
                console.log('unregister request failed: %s', err.message)
            } else {
                console.log('store %s removed', resource)
            }
        });
    }

    return {
        register  : register,       //done
        create    : create,         //done
        activate  : activate,       //done
        deactivate: deactivate,     //done
        setStatus : setStatus,      //done
        list      : list,           //done
        fetch     : fetch,          //done
        link      : link,           //done
        unlink    : unlink,         //done
        tag       : tag,            //done
        untag     : untag,          //done
        unregister: unregister      //done
    }

};
